

Proposed Changes to the Transformation API
==========================================

Yudi previously added some useful macros for annotation of state to be
transferred:

MIGRATE_GLOBAL(g)

and 

MIGRATE_LOCAL(l)

These were intended to be useful for annotating state that only needs
to be memcpy'd between versions and doesn't require transformation.
When transformation is required, we fallback to annotations like the
ones we've used previously that specify the variable and the
transformation function.

One shortcoming of this is that we are hard-coding the transformation
(or lack of transformation) that will be used into the original source
files.  So far in the Kitsune work, we've tried to keep the particular
code for transformation in a separate source file (dsu.c) so that we
could use maintain multiple versions of it (i.e., v0->v2, v1->v2,
v2->v2).  We also keep it separate because we're building a tool to
generate it and it would be nice to let that tool stick all of its
generated code together.

I've proposed the following change to Ted (and Yudi, previously) and
Ted has agreed to implement it:

The MIGRATE_GLOBAL and MIGRATE_LOCAL macros will support
transformation.  They will determine the transformation function to be
used from the arguments that they receive.  For example:

     MIGRATE_GLOBAL(g) calls __kitsune_transform_global_g()

     MIGRATE_LOCAL(l)  calls __kitsune_transform_local_foo_l()  
               (if MIGRATE_LOCAL is called from foo)

These mangled names will be the ones used in the dsu.c file to
implement the transformation.  New macros, GLOBAL_XFORM_NAME(g) and
LOCAL_XFORM_NAME(fun, local), can be used by the developer to name
their transformation functions accordingly.

A nice trick is that, the code generated by MIGRATE_(LOCAL|GLOBAL) can
look in the new versions' symbol table for a function of the correct
name.  If it does not exist, the code can fall-back to the default
memcpy behavior that Yudi has already implemented.

The beauty of this is that developers don't need to write any trivial
"identity" transformation functions in cases where complex
transformation is not needed (for many programs, we expect this to be
the common case).  When they are needed, they should be relatively
straightforward to add.

In addition, the transformation generation tool can generate functions
with the neccessary names when they are needed, so it will dovetail
cleanly with this approach.




