Tor Experience (2011/09/16)
==============

#  Questions

* We tagged over 50 global state items (just for onion proxy update
support at the moment). Should we be concerned about the difficulty of
this?

No. The relevant state was already tagged with Doxygen comments;
so it took very little work to determine which state to update. All of
the state in Tor (as far as we are aware) is tagged, but this wasn't
difficult. 

* Many state items were (originally) initialized at their global
definitions using static initializers. As a result, we created
module-level migration functions that are called at startup and
migrate a bunch of the globals in that file. Perhaps there would be
benefit in adding a global variable annotation that the compiler could
use to automatically generate these module-level functions?

Yes, this would definitely be useful. 

* Currently we have no local variable state items tagged (and
apparantly don't expect to need to need any). Was this choice obvious
or are there any tradeoffs being made?

Tor is very event-driven, and so it rarely has a long call stack.
However, there was one occasion where a static local variable was
refactored into a global (at the time, there was no other
functionality for updating locals). It was the array of signal events.
Now, the change that was introduced could probably be replaced with
NOTE_AND_MIGRATE_LOCAL_STATIC.

* Tor uses a "subtyping" idiom for some of its data. This means that
casting a data item to a parent type hides some of its data. How does
it indicate what concrete type is stored in each pointer to the parent
type that it maintains? Does this idiom complicate state
transformation? If it hasn't yet, might it once more Tor functionality
is supported?

For all versions in our streak, it doesn't appear that this will ever
complicate state transformation. The mapping is fully reversible; the
base types always define a field that contains the object's type.


* How complex/numerous were the control-flow changes required to get
Tor back to its main loop after an update?

Tor goes back to its main loop just fine; since it's purely
event-driven, the path to the mainloop is short. Aside from state
migration, there are no control flow changes. 

## Harder questions

* Currently we have a bunch of cases where a global variable
   initializer has both a MIGRATE_GLOBAL and "if
   (!kitsune_is_updating())" block. We probably need to merge these. Is
   there a reason not to merge them in any of the cases?
* Tor uses threads in a limited way. We have determined that this use
   doesn't require any special effort on our part. Can you summarize
   why this is? Are there any cases where the threads could pose
   challenges?

* Are we rereading configuration files at each version's startup? Why
   was this decision made?

No. This would require inserting update checks into significantly more
code. Skipping this is simpler, and since Tor defines a controller and
UNIX signal for reloading its configuration, reloading configuration
files is unnecessary.

#  Notes

Tor's particular way of using libevent causes us to encounter
update points only in certain rare circumstances, including soon after
an update has been requested. This would make deterministic testing as
we did in our DSUTest work difficult. We'll probably just initiate an
update point approximately (e.g., sometime during a download, or
sometime while tor is setting up its network configuration).

#  Summer Updating Notes

Over the summer we built a streak of twelve versions of Tor, from the
"stable" development branch. Although the delta between each
individual version in the streak is small, problems associated with
debugging aspects of Tor and the libraries that Tor uses (namely
libevent) caused the streak to take a long time.

Tor is a very stateful program. During an execution, it maintains
lists of connections, "circuits", and router descriptors; lists of
specialized routers (such as bridge and guard nodes); TLS contexts,
encryption keys, and directory server certificates; and a myriad of
accounting information. In the first version (0.2.1.18, or v0), 244
items of state were tagged with a MIGRATE_* kitsune macro, and that
number grew to 339 by v12. Most of this tagging code could have easily
been automatically generated, and was typically present in a single
function per module, named migrate_<modulename>. While most of the
variables tagged for updating were referenced in the Tor doxygen
documentation, some weren't, and these variables caused bugs on
occasion before they were discovered and updated.

The main problems that were involved with updating Tor pertained to
debugging Tor while it updated, and updating Tor while it used the
LibEvent library. These problems compounded each other, because
libevent failures typically were sporadic and non-deterministic,
causing them to interfere with the already-tenuous process of
debugging other issues in Tor, while it was nearly impossible to debug
Tor's use of libevent.

Libevent is a library that is designed to replace the main event loop
of a program. Client programs register callbacks for events such as
socket buffer availability, signals, and timers, and libevent monitors
for those events and executes callbacks. Callback functions passed
into libevent prior to an update will expire after an update, so to
update Tor, it was necessary to modify LibEvent's state. We found
three strategies for doing this:
 
* Deleting the libevent "event base" that held all pertinent libevent
  state upon an update, and then re-inserting all events into a newly
  created event base
* Clearing the libevent event base with the event_reinit function, and
  then re-inserting all events to the old (but wiped) event base
* Manually deleting all events and re-adding them
* Modifying the event structures on the heap to change their callback
  function address.
  
Due to a non-deterministic bug in its interactions with the kernel
Linux's epoll mechanism, we erroneously ruled out option 4, even
though it is the simplest and least error-prone option. For a long
time during the development of the v0-v0 and v0-v1 update, we used
option 2. However, this strategy proved to not work reliably for all
libevent backends, nor did it allow us to update more than once.
Option 1 non-deterministically caused either an assertion failure or
segfault inside libevent, due to a bug in libevent (caused by libevent
not deleting certain events). Finally, we switched to option 4, and
configured our environment to prefer the poll(2) backend for libevent,
rather than epoll(2). 

Diagnosing and solving this problem was made more difficult by gdb's
inability to debug programs during an kitsune update. During an kitsune
update, the new version of the code is running through the program's
initialization, such that the update stage ends once control reaches
the update point again. During this stage, both the old and new
versions of the code are loaded into memory, allowing global variables
to be copied into the new address space. However, gdb will only break
execution for a breakpoint on the old address. This makes gdb useless
for pre-mortem debugging during an update. While a backtrace will be
displayed and the developer can diagnose at the point of failure, the
only way to set breakpoints at this stage of execution is to break
outside of the main program shared object (such as inside the kitsune
runtime, or in Tor's case, in libevent).


We plan to run a Tor router in the wild to test our streak and ability
to adapt to new versions of Tor. To do this, we first need to extend
our streak from the stable to the development branch of Tor. This in
itself is a larger update that rivals our entire existing streak in
size. After that, we plan to aggressively integrate all Tor
development releases (occurring at a frequency of every few weeks).
Our router will carry the encrypted traffic of actual Tor users. Since
Tor nodes are penalized for downtime in the reputation metrics that
determine directory server flags and client preference, we expect this
dynamically updated node to achieve those flags faster and carry more
traffic than a node that is restarted every time it must upgrade to
the new version.
